// Candle Clear by Gumphrie v0.3
// Candle Clear by Gumphrie v0.4 - now with outline and wick colouring options
// Candle Clear by Gumphrie v0.5 - fixed pixel out by one problem at bottom of wicks

#region Using declarations
using System;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.ComponentModel;
using System.Xml.Serialization;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.Design;
using NinjaTrader.Cbi;
#endregion

// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    /// <summary>
	/// Candle Clear
    /// </summary>
    [Description("Candle Clear")]
	[Gui.Design.DisplayName("Candle Clear")]
	public class CandleClear : Indicator
    {
		
        #region Variables
		private DataSeries 	open;
		private DataSeries 	high;
		private DataSeries 	low;
		private DataSeries 	close;
		
		private bool thickWick=false;
		
		private Color wickColorUp 	= Color.Black;
		private Color wickColorDown 	= Color.Black;
		private Color outlineColorUp 	= Color.Black;
		private Color outlineColorDown 	= Color.Black;
		#endregion

        /// <summary>
        /// This method is used to configure the indicator and is called once before any bar data is loaded.
        /// </summary>
        protected override void Initialize()
        {
			open  = new DataSeries(this);
			high  = new DataSeries(this);
			low   = new DataSeries(this);
			close = new DataSeries(this);
			
			PlotsConfigurable 	= false;
			PaintPriceMarkers 	= false;
			Overlay				= true;		
			CalculateOnBarClose = false;
			
        }

        /// <summary>
        /// Called on each bar update event (incoming tick)
        /// </summary>
        protected override void OnBarUpdate()
        {			
			if (CurrentBar == 0)
			{
				open.Set(0);
				high.Set(0);
				low.Set(0);
				close.Set(0);
				return;
			}
			
           
			open.Set(Open[0]);
			high.Set(High[0]);
			low.Set(Low[0]);
			close.Set(Close[0]);
			
			if (ChartControl.ChartStyleType.ToString()=="CandleStick")
			{
				if (Close[0]>Open[0])
				{
					BarColor=ChartControl.ChartStyle.UpColor;
				}
				else
				{
					BarColor=ChartControl.ChartStyle.DownColor;
				}
			}
			
		}

		public override void Plot(Graphics graphics, Rectangle bounds, double min, double max)
		{
			// Default plotting in base class. 
			//base.Plot(graphics, bounds, min, max);
			
			if (base.Bars == null) return;
			
			if (ChartControl.ChartStyleType.ToString()!="CandleStick") return;
			
        	int index = -1;
			Exception caughtException;
			
				
            int barPaintWidth = base.ChartControl.ChartStyle.GetBarPaintWidth(base.ChartControl.BarWidth);
			int bars = base.ChartControl.BarsPainted;
            while (bars >= 0)
            {
				index = ((base.ChartControl.LastBarPainted - base.ChartControl.BarsPainted) + 1) + bars;
                if (base.ChartControl.ShowBarsRequired || ((index - base.Displacement) >= base.BarsRequired))
                {
					try
					{
						int x1 = (((base.ChartControl.CanvasRight - base.ChartControl.BarMarginRight) - (barPaintWidth / 2)) - ((base.ChartControl.BarsPainted - 1) * base.ChartControl.BarSpace)) + (bars * base.ChartControl.BarSpace);
						int y1 = (bounds.Y + bounds.Height) - ((int) (((high.Get(index) - min) / ChartControl.MaxMinusMin(max, min)) * bounds.Height));
						int y2 = (bounds.Y + bounds.Height) - ((int) (((close.Get(index) - min) / ChartControl.MaxMinusMin(max, min)) * bounds.Height));
						int y3 = (bounds.Y + bounds.Height) - ((int) (((open.Get(index) - min) / ChartControl.MaxMinusMin(max, min)) * bounds.Height));
                        int y4 = (bounds.Y + bounds.Height) - ((int) (((low.Get(index) - min) / ChartControl.MaxMinusMin(max, min)) * bounds.Height));
                 		
						y1--;
						y2--;
						y3--;
						//y4--;
						
            			Pen pen;
						Pen wickpen;
							
						if (y2>y3)
						{
						    if (thickWick) wickpen = new Pen(wickColorDown, 3);
						    else wickpen = new Pen(wickColorDown, 1);
							pen = new Pen(outlineColorDown, 1);
						
							graphics.DrawLine(wickpen,x1,y1,x1,y3);
							graphics.DrawLine(wickpen,x1,y2,x1,y4);
							graphics.FillRectangle(new SolidBrush(ChartControl.ChartStyle.DownColor),x1-(barPaintWidth/2),y3,barPaintWidth,y2-y3);
						//	graphics.FillRectangle(new SolidBrush(Color.Red),x1-(barPaintWidth/2),y3,barPaintWidth,y2-y3);
						}
						else
						{
							if (thickWick) wickpen = new Pen(wickColorUp, 3);
						    else wickpen = new Pen(wickColorUp, 1);
							pen = new Pen(outlineColorUp, 1);
							
							graphics.DrawLine(wickpen,x1,y1,x1,y2);
							graphics.DrawLine(wickpen,x1,y3,x1,y4);
							graphics.FillRectangle(new SolidBrush(ChartControl.ChartStyle.UpColor),x1-(barPaintWidth/2),y2,barPaintWidth,y3-y2);
						}
                        graphics.DrawLine(pen,x1-(barPaintWidth/2),y3,x1+(barPaintWidth/2),y3);
						graphics.DrawLine(pen,x1-(barPaintWidth/2),y2,x1+(barPaintWidth/2),y2);
						graphics.DrawLine(pen,x1-(barPaintWidth/2),y2,x1-(barPaintWidth/2),y3);
						graphics.DrawLine(pen,x1+(barPaintWidth/2),y2,x1+(barPaintWidth/2),y3);
					}
					catch (Exception exception) { caughtException = exception; }
               	}
				bars--;					
			}
		}

        #region Properties
		
		/// <summary>
		/// </summary>
		[Description("Whether to display a thick wick")]
		[Category("Parameters")]
		[NinjaTrader.Gui.Design.DisplayName("Thick wick")]
		public bool ThickWick
        {
            get { return thickWick; }
			set { thickWick = value; }
        }
		
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		public string WickColorDownSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(wickColorDown); }
			set { wickColorDown = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		public string WickColorUpSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(wickColorUp); }
			set { wickColorUp = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		public string OutlineColorDownSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(outlineColorDown); }
			set { outlineColorDown = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		/// <summary>
		/// </summary>
		[Browsable(false)]
		public string OutlineColorUpSerialize
		{
			get { return NinjaTrader.Gui.Design.SerializableColor.ToString(outlineColorUp); }
			set { outlineColorUp = NinjaTrader.Gui.Design.SerializableColor.FromString(value); }
		}
		
		/// <summary>
		/// </summary>
		[Description("Wick Down color."),XmlIgnore, VisualizationOnly]
		[Category("Plot")]
		[Gui.Design.DisplayNameAttribute("Wick Down color")]
		public Color WickColorDown
		{
			get { return wickColorDown; }
			set { wickColorDown = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("Wick Up color."),XmlIgnore, VisualizationOnly]
		[Category("Plot")]
		[Gui.Design.DisplayNameAttribute("Wick Up color")]
		public Color WickColorUp
		{
			get { return wickColorUp; }
			set { wickColorUp = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("Outline Down color."),XmlIgnore, VisualizationOnly]
		[Category("Plot")]
		[Gui.Design.DisplayNameAttribute("Outline Down color")]
		public Color OutlineColorDown
		{
			get { return outlineColorDown; }
			set { outlineColorDown = value; }
		}
		
		/// <summary>
		/// </summary>
		[Description("Outline Up color."),XmlIgnore, VisualizationOnly]
		[Category("Plot")]
		[Gui.Design.DisplayNameAttribute("Outline Up color")]
		public Color OutlineColorUp
		{
			get { return outlineColorUp; }
			set { outlineColorUp = value; }
		}
		
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private CandleClear[] cacheCandleClear = null;

        private static CandleClear checkCandleClear = new CandleClear();

        /// <summary>
        /// Candle Clear
        /// </summary>
        /// <returns></returns>
        public CandleClear CandleClear(bool thickWick)
        {
            return CandleClear(Input, thickWick);
        }

        /// <summary>
        /// Candle Clear
        /// </summary>
        /// <returns></returns>
        public CandleClear CandleClear(Data.IDataSeries input, bool thickWick)
        {
            checkCandleClear.ThickWick = thickWick;
            thickWick = checkCandleClear.ThickWick;

            if (cacheCandleClear != null)
                for (int idx = 0; idx < cacheCandleClear.Length; idx++)
                    if (cacheCandleClear[idx].ThickWick == thickWick && cacheCandleClear[idx].EqualsInput(input))
                        return cacheCandleClear[idx];

            CandleClear indicator = new CandleClear();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.ThickWick = thickWick;
            indicator.SetUp();

            CandleClear[] tmp = new CandleClear[cacheCandleClear == null ? 1 : cacheCandleClear.Length + 1];
            if (cacheCandleClear != null)
                cacheCandleClear.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheCandleClear = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// Candle Clear
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.CandleClear CandleClear(bool thickWick)
        {
            return _indicator.CandleClear(Input, thickWick);
        }

        /// <summary>
        /// Candle Clear
        /// </summary>
        /// <returns></returns>
        public Indicator.CandleClear CandleClear(Data.IDataSeries input, bool thickWick)
        {
            return _indicator.CandleClear(input, thickWick);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// Candle Clear
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.CandleClear CandleClear(bool thickWick)
        {
            return _indicator.CandleClear(Input, thickWick);
        }

        /// <summary>
        /// Candle Clear
        /// </summary>
        /// <returns></returns>
        public Indicator.CandleClear CandleClear(Data.IDataSeries input, bool thickWick)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.CandleClear(input, thickWick);
        }

    }
}
#endregion
